"""
Skill implementation of spacy named entity extraction.
The spacy model can be configured by the env property SPACY_MODEL.
"""

# Skill calls will begin at the evaluate() method, with the skill payload passed as 'payload'
# and the skill context passed as 'context'.
# For more details, see TODO: url
def evaluate(payload: dict, context: dict) -> dict:
    print(payload)
    print(context)
    message = payload["request"]
    return {"response" : "ECHO (from={}/{}): {}".format(context["owner"], context["name"], message)}

def on_started(context: dict):
    print("on_started triggered!")

def on_stopped(context: dict):
    print("on_stopped triggered!")


import sys


class ExitCodeException(Exception):
    """
    Base class for all exceptions that set the exit code
    """
    def get_exit_code(self):
        """ Override in derived classes """
        return 1


def handle_uncaught_exception(exception_type, value, trace):
    old_hook(exception_type, value, trace)
    if isinstance(value, ExitCodeException):
        sys.exit(value.get_exit_code())


sys.excepthook, old_hook = handle_uncaught_exception, sys.excepthook


class LADMError(ExitCodeException):
    def get_exit_code(self):
        return 5



def get_allowed_system_dependencies():
    allowed_system_dependencies = [
        "libffi-dev",
        "default-jre",
        "openjdk-11-jre-headless",
        "openjdk-8-jre-headless"
    ]
    return allowed_system_dependencies



import shutil
import unittest
from pathlib import Path
from ladm.main.main import LADM
from ladm.main.util.custom_exceptions import LADMError


class LADMTestCase(unittest.TestCase):
    default_output_path = "test-output/LADM"
    default_output_file_name = "LADM-requirements.txt"

    def tearDown(self) -> None:
        if Path(self.default_output_path).exists():
            shutil.rmtree(self.default_output_path.split('/')[0])

    def test_correct_illegal_dependency_filtering(self) -> None:
        dependencies_yml_path = "resources/correct-skill-ladm/dependencies.yml"
        ladm = LADM(dependencies_yml_path,
                    "python", str(Path(self.default_output_path) / self.default_output_file_name))
        results = ladm.generate_dependencies_from_file()
        self.assertTrue((Path(self.default_output_path) / self.default_output_file_name).exists())

        # Illegal dependencies are in dependencies.yml
        self.assertTrue(all(
            [illegal_dependency in Path(dependencies_yml_path).read_text() for illegal_dependency in ["vim", "emacs"]]))
        # Are correctly filtered out
        self.assertEqual((["zupline==1.0.0", "tensorblow"],
                          ["libffi-dev", "default-jre=2:1.11-71"],
                          []), results)

    def test_missing_version(self) -> None:
        ladm = LADM("resources/incorrect-skill-ladm/dependencies_missing_version.yml",
                    "python")
        self.assertRaises(LADMError, ladm.generate_dependencies_from_file)

    def test_empty_dependencies(self) -> None:
        ladm = LADM("resources/incorrect-skill-ladm/dependencies_empty.yml",
                    "python")
        self.assertRaises(LADMError, ladm.generate_dependencies_from_file)

    def test_correct_models(self) -> None:
        dependencies_yml_path = "resources/correct-skill-ladm/dependencies_with_models.yml"
        ladm = LADM(dependencies_yml_path,
                    "python", str(Path(self.default_output_path) / self.default_output_file_name))
        results = ladm.generate_dependencies_from_file()
        self.assertEqual((["zupline==1.0.0", "tensorblow"],
                          ["libffi-dev", "default-jre=2:1.11-71"],
                          ["gaia://tenant/a.txt", "gaia://tenant/test/b.txt"]), results)

    def test_correct_wrong_format_model_filtering(self) -> None:
        dependencies_yml_path = "resources/incorrect-skill-ladm/dependencies_with_models_wrong_format.yml"
        ladm = LADM(dependencies_yml_path,
                    "python", str(Path(self.default_output_path) / self.default_output_file_name))
        results = ladm.generate_dependencies_from_file()

        # Models with wrong format are in dependencies.yml
        self.assertTrue(all(
            [illegal_dependency in Path(dependencies_yml_path).read_text() for illegal_dependency in
             ["s3://tenant/test/a.txt", "gaia://tenant"]]))
        # Are correctly filtered out
        self.assertEqual((["zupline==1.0.0", "tensorblow"],
                          ["libffi-dev", "default-jre=2:1.11-71"],
                          ["gaia://tenant/a.txt", "gaia://tenant/test/b.txt"]), results)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ladm']

package_data = \
{'': ['*'],
 'ladm': ['main/*',
          'main/util/*',
          'tests/*',
          'tests/resources/correct-skill-ladm/*',
          'tests/resources/correct-skill-ladm/src/*',
          'tests/resources/correct-skill-ladm/src/contract/*',
          'tests/resources/incorrect-skill-ladm/*',
          'tests/resources/incorrect-skill-ladm/src/*',
          'tests/resources/incorrect-skill-ladm/src/contract/*']}

install_requires = \
['pyyaml>=5.3.1,<6.0.0']

entry_points = \
{'console_scripts': ['test = poetry_scripts:test']}

setup_kwargs = {
    'name': 'ladm',
    'version': '0.0.1',
    'description': 'Language Agnostic Dependency Management',
    'long_description': None,
    'author': 'Leftshift One',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.7,<4.0.0',
}


setup(**setup_kwargs)
